package organizer.components;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

import organizer.images.Images;
import organizer.messages.Messages;

import echopointng.DateChooser;
import echopointng.model.CalendarEvent;
import echopointng.model.CalendarSelectionListener;
import echopointng.model.CalendarSelectionModel;
import echopointng.model.DefaultCalendarSelectionModel;
import nextapp.echo2.app.ApplicationInstance;
import nextapp.echo2.app.Button;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Row;
import nextapp.echo2.app.SplitPane;
import nextapp.echo2.app.TextField;
import nextapp.echo2.app.WindowPane;
import nextapp.echo2.app.event.ActionEvent;
import nextapp.echo2.app.event.ActionListener;
/**
 * Trieda DateWindow sli ako okno pre mal kalendrik. Uahuje uvateovi zadvanie dtumu. 
 * Kalendrik je predstavovan triedov DateChooser kninice echopointng. Pre pouitie triedy je
 * nutn poui jednoparametrick kontruktor, ktor preber ako parameter objekt typu TextField.
 * Do tohto textovho poa je nsledne vloen hodnota z kalendra. 
 *
 */
public class DateWindow extends WindowPane
{
	/** SimpleDateFormat sli na formtovanie dtumu vyberanho z kalendra. */
	private SimpleDateFormat dateFormat = new SimpleDateFormat(Messages.getMessage("DateFormat"), ApplicationInstance.getActive().getLocale());
	/** Je odkaz na textov pole, do ktorho bude vloen dtum. */
	private TextField field;
	/** Odkaz na dtum, ktor bude zapsan do textovho poa. */
	private Date selectedDate;
	
	/**
	 * Jednoparametrick kontruktor, ktor my budeme pouva ako hlavn kontruktor pre 
	 * prcu s touto triedou. Kontruktor vytvor okno, ktormu podobne ako v prpade triedy
	 * AddWindow nastav vlastnosti. Po nadefinovan zkladou vytvor objekt DateChooser spolu
	 * s potrednmi objektami. Nakoniec je vetko pridan spolu s odosielacm tlaidlo do 
	 * objektu typu SplitPane, ktor je pridny do samotnho okna. 
	 * 
	 * @param field Je textov pole, do ktorho bude vloen dtum.
	 */
	public DateWindow(TextField field)
	{
		// Nastavme zkladn vlastnosti
		this.field = field;
		setStyleName("FormWindow");
	    setHeight(new Extent(315));
	    setWidth(new Extent(207));
	    setModal(true);
	    // Vytvorme SplitPane	    
	    SplitPane sp = new SplitPane(SplitPane.ORIENTATION_VERTICAL_BOTTOM_TOP,new Extent(30));
	    /*==========================VYTVRANIE KOMPONENTOV==========================*/
	    //  Najskr vytvorme pre objekt triedy DateChooser model. 
	    Calendar cal = Calendar.getInstance();
	    /* 
	     * Pri vytvaran kalendrika skontrolujeme, i je u zadan dtum v textovom poli. 
	     * Pokia no tak ho ozname v kalendri. Pokial nebol zadan odignorujeme vnijmku. 
	     */
	    try 
	    {
	         Date date = dateFormat.parse(field.getText());
	         cal.setTime(date);
	    } catch (ParseException ex) {
	    	  // vyhod sa pokia nie je zadan iadny dtum nie je potrebn oetri.
	    }
	    CalendarSelectionModel model = new DefaultCalendarSelectionModel(cal);
	    // Pripojme listenera
	    model.addListener(listener);
	    // Date chooser - samotny kalendarik
	    DateChooser dateChooser = new DateChooser();
	    dateChooser.setModel(model);
	    dateChooser.setStyleName("DateChooser");
		// Odosielacie tlaidlo
		Button submitButton = new Button();
	    submitButton.setIcon(Images.SUBMIT_BUTTON);
	    submitButton.addActionListener(buttonListener);
	    // Riadok pre odosielacie tlaidlo
	    Row submitRow = new Row();
	    submitRow.setStyleName("SubmitButtonRow");
	    /*==========================PRIDVANIE KOMPONENTOV==========================*/
		// Riadok pre odosielacie tlaidlo
	    submitRow.add(submitButton);
	    // SplitPane
	    sp.add(submitRow);
	    sp.add(dateChooser);
	    
	    add(sp);
 
	    ApplicationInstance.getActive().getDefaultWindow().getContent().add(this);  
	}
/*==========================LISTENERS==========================*/
	/**
	 * Listener slui pre potvrdenie vberu dtumu. Vybran dtum je nsledne vpsan do 
	 * textovho poa.
	 * */
	private ActionListener buttonListener = new ActionListener()
	{
		public void actionPerformed(ActionEvent ae)
		{
			String dateString = dateFormat.format(selectedDate);
			field.setText(dateString);
			DateWindow.this.getParent().remove(DateWindow.this);
		}
	};
	
	/**
	 * Listener, ktor je napojen na model dtumu. Jednotliv metdy vyberaj a ukladaj 
	 * vybran dtum do atribtu.
	 * */
	private CalendarSelectionListener listener = new CalendarSelectionListener() 
	{
		public void displayedDateChange(CalendarEvent ce) 
		{
			CalendarSelectionModel model = (CalendarSelectionModel)ce.getSource();
			selectedDate = model.getSelectedDate().getTime();
		}

		public void selectedDateChange(CalendarEvent ce) 
		{
			CalendarSelectionModel model = (CalendarSelectionModel)ce.getSource();
			selectedDate = model.getSelectedDate().getTime();	
		}
	};
}
	