package organizer.components;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collection;

import nextapp.echo2.app.Alignment;
import nextapp.echo2.app.Border;
import nextapp.echo2.app.Color;
import nextapp.echo2.app.Component;
import nextapp.echo2.app.Extent;
import nextapp.echo2.app.Insets;
import nextapp.echo2.app.Label;
import nextapp.echo2.app.Table;
import nextapp.echo2.app.layout.TableLayoutData;
import nextapp.echo2.app.table.AbstractTableModel;
import nextapp.echo2.app.table.DefaultTableColumnModel;
import nextapp.echo2.app.table.TableCellRenderer;
import nextapp.echo2.app.table.TableColumn;
import nextapp.echo2.app.table.TableColumnModel;

import organizer.dto.DTOTask;
import organizer.messages.Messages;

/**
 * Trieda sli na zobrazenie loh ako aj infomcii o nich. Jednotliv lohy s zobrazen 
 * v tabuke. 
 * 
 * @author Peter Toth
 *
 */
public class TaskTable extends Table
{
	private AbstractTableModel tableModel;
	private TableLayoutData cellLayout;
	private TableLayoutData headerLayout;
	SimpleDateFormat formater = new SimpleDateFormat(Messages.getMessage("DateFormat"));
	
	/**
	 * Kontruktor, ktor vytvra zklad celej tabuky. Kee n projekt nie je napojen
	 * na databzy jednotliv lohy vytvrame rune.
	 * 
	 * @param tasks Kolekcia loh, ktor sa maj zobrazi. 
	 */
	public TaskTable(Collection<DTOTask>tasks)
	{
		// Layout pre bunky tabuky
		cellLayout =  new TableLayoutData();
  	  	cellLayout.setAlignment(Alignment.ALIGN_CENTER);
  	  	cellLayout.setBackground(new Color(0x3366cc));
  	  	cellLayout.setInsets(new Insets(new Extent(3)));
  	  	// Layout pre hlaviku tabuky
  	  	headerLayout = new TableLayoutData();
  	  	headerLayout.setAlignment(Alignment.ALIGN_CENTER);
  	  	headerLayout.setBackground(new Color(0xeeeeee));
  	    headerLayout.setInsets(new Insets(new Extent(3)));
  	  	// Nastavme model pre dta a stpce tabuky
  	  	tableModel = new TaskTableModel(tasks);
  	  	setModel(tableModel);
  	  	setColumnModel(createTabColModel());
  	  	// 	Nastavme vzhad tabuky.
 	  	setDefaultRenderer(Object.class,cellRenderer);
 	  	setDefaultHeaderRenderer(headCellRenderer);
 	  	
 	  	setWidth(new Extent(100,Extent.PERCENT));
 	  	setBorder(new Border(new Extent(1),Color.BLACK,Border.STYLE_SOLID));
	}
	
	/**
	 * Vntorn trieda predstavujca dtov model tabuky. Dtov model pracuje s dtami, ktor
	 * sa maj zobrazi v tabuke. 
	 * 
	 */
	private class TaskTableModel extends AbstractTableModel 
	{	  
	  public static final int COLUMN_COUNT = 7;
	  private final String[] headerNames = new String[] 
	  {
			  Messages.getMessage("TaskTable.Name"),
			  Messages.getMessage("TaskTable.Start"),
			  Messages.getMessage("TaskTable.End"),
			  Messages.getMessage("TaskTable.Priority"),
			  Messages.getMessage("TaskTable.State"),
			  Messages.getMessage("TaskTable.Alarm"),
			  Messages.getMessage("TaskTable.Text")
	  };
	  private ArrayList<DTOTask>tasks = new ArrayList<DTOTask>();		
	  
	  
	  /**
	   * Kontruktor preberie a ulo dta.
	   * 
	   * @param tasks lohy, s ktormi bude model pracova.
	   */
	  public TaskTableModel(Collection<DTOTask>tasks)
	  {
		  this.tasks.addAll(tasks);
	  }
	  
	  /**
	   * Metod vracia poet stpcov tabuky.
	   */
	  public int getColumnCount() {
	    return COLUMN_COUNT;
	  }

      /**
       * Metod vracia poet riadkov tabuky, teda poet zznamov v kolekcii. 
       */
	  public int getRowCount() {
	    return tasks.size();
	  }
	  
	  /**
	   * Metda vracia hodnotu z uritho riadku a stpca. Poda toho,ktor stpec
	   * vykreslujeme vrtime jeden z atribtov triedy.
	   */
	  public Object getValueAt(int colIndex, int rowIndex) 
	  {
		  DTOTask task = tasks.get(rowIndex);
		  switch (colIndex) 
		  {
	      	case 0:
	      		return task.getName();
			case 1:
				return formater.format(task.getStartTime());
			case 2:
				return formater.format(task.getEndTime());
			case 3:
				String isInProgres;
				if(task.isInProgress())
					isInProgres = Messages.getMessage("TaskTab.State.Working");
				else 
					isInProgres = Messages.getMessage("TaskTab.State.Preparation");
				return isInProgres;
			case 4:
				return ""+task.getPriority();
			case 5:
				String alarm;
				if(task.isAlarm())
					alarm = Messages.getMessage("General.Yes");
				else
					alarm = Messages.getMessage("General.No");
				return alarm;
			case 6: 
				return task.getTaskText();
				default:
				return "";
		  }
	}
	  
	/**
	 * Metod vracajca zoznam loh, s ktormi model pracuje.
	 * 	
  	 * @return Koleciu objektov triedy DTOTask.
  	*/
	public ArrayList<DTOTask>getTasks()
	{
		return tasks;
	} 
	
	/**
	 * Nepovinn prekryt metda. Metod vracia men jednotlivch stpcov, a to tak,
	 * e ta hodnoty pol.
	 * 
	 */  public String getColumnName(int colIndex) 
    {
			return headerNames[colIndex];
	}
  

	}
	
	/**
	 * Objekt typu TableCellRendered, ktor sli na vykreslenie buniek v tabuke. 
	 */
	private TableCellRenderer cellRenderer = new TableCellRenderer() 
	{
		  /**
		   * Metda, ktorou sa vykresluj jednotliv bunky.
		   */
		  public Component getTableCellRendererComponent(Table table,Object value, int column, int row)
		  {
		    Label cellLabel = new Label((String) value);
			cellLabel.setForeground(new Color(0x3366cc));
			cellLabel.setLayoutData(headerLayout);
			return cellLabel;
		  }
	};
		
	/**
	 * Objekt typu TableCellRendered, ktor sli na vykreslenie hlaviky v tabuke. 
	 */
	private  TableCellRenderer headCellRenderer = new TableCellRenderer() 
	{
		/**
		 * Metda, ktorou sa vykresluje hlavika.
		 */
		public Component getTableCellRendererComponent(Table tab, Object value,	int col, int row) 
		{
			Label headerLabel = new Label((String) value);
			headerLabel.setForeground(new Color(0xffffff));
			headerLabel.setLayoutData(cellLayout);
			return headerLabel;
		}
	};
	
	/*==========================Class own methods==========================*/

	/**
	 * Metda vytvorme stpcov model tabuky.
	 * 
	 * @return Stpcov model tabuky.
	 */
	private TableColumnModel createTabColModel()
	{
		TableColumnModel tabColModel = new DefaultTableColumnModel();
		TableColumn[] tabCols = createTabCols();
		tabColModel.addColumn(tabCols[0]);
		tabColModel.addColumn(tabCols[1]);
		tabColModel.addColumn(tabCols[2]);
		tabColModel.addColumn(tabCols[3]);
		tabColModel.addColumn(tabCols[4]);
		tabColModel.addColumn(tabCols[5]);
		tabColModel.addColumn(tabCols[6]);
		return tabColModel;
	}

	/**
	 * Metda vytvor pole s objektami predstavujce stpce.
	 * 
	 * @return Pole objektov predstavujcih stpce.
	 */
	private TableColumn[] createTabCols() {
		TableColumn[] tabCols = new TableColumn[7];
		tabCols[0] = new TableColumn(0, new Extent(8, Extent.PERCENT),
				cellRenderer, headCellRenderer);
		tabCols[1] = new TableColumn(1, new Extent(5, Extent.PERCENT),
				cellRenderer, headCellRenderer);
		tabCols[2] = new TableColumn(2, new Extent(5, Extent.PERCENT),
				cellRenderer, headCellRenderer);
		tabCols[3] = new TableColumn(3, new Extent(8, Extent.PERCENT),
				cellRenderer, headCellRenderer);
		tabCols[4] = new TableColumn(4, new Extent(10, Extent.PERCENT),
				cellRenderer, headCellRenderer);
		tabCols[5] = new TableColumn(5, new Extent(5, Extent.PERCENT),
				cellRenderer, headCellRenderer);
		tabCols[6] = new TableColumn(6, new Extent(59, Extent.PERCENT),
				cellRenderer, headCellRenderer);
		return tabCols;
	}
}
